#include <exception>
using namespace std;

#include "Status.h"

/**
* La nostra unica reference a Status
*/
Status* Status::Instance = NULL;

Status* Status::self()
{
	if(Instance == NULL)
		Instance = new Status();

	return Instance;
}

Status::Status() : bActionsLock(INVALID_HANDLE_VALUE), bAgentsLock(INVALID_HANDLE_VALUE), bConfigLock(INVALID_HANDLE_VALUE), 
bEventsLock(INVALID_HANDLE_VALUE), agents(NULL), events(NULL), actions(NULL), config(NULL), device(NULL) {
	bActionsLock = CreateMutex(NULL, FALSE, NULL);
	bAgentsLock = CreateMutex(NULL, FALSE, NULL);
	bEventsLock = CreateMutex(NULL, FALSE, NULL);
	bConfigLock = CreateMutex(NULL, FALSE, NULL);
}

Status::~Status() {
	CloseHandle(bActionsLock);
	CloseHandle(bAgentsLock);
	CloseHandle(bEventsLock);
	CloseHandle(bConfigLock);
}

// Chi fa la get* dovrebbe controllare e lockare i relativi semafori
list<AgentStruct*>* Status::GetAgentsList() {
	return agents;
}

list<EventStruct*>* Status::GetEventsList() {
	return events;
}

list<MacroActionStruct*>* Status::GetActionsList() {
	return actions;
}

list<ConfStruct*>* Status::GetConfigStruct() {
	return config;
}

DeviceStruct* Status::GetDeviceStruct() {
	return device;
}

BOOL Status::AddAgent(AgentStruct* pAgent) {
	AgentStruct *newAgent = NULL;
	list<AgentStruct *>::iterator iter;

	// Gestiamo l'accesso concorrente
	WAIT_AND_SIGNAL(bAgentsLock);
	if(agents == NULL){
		agents = new(std::nothrow) list<AgentStruct *>;

		if(agents == NULL){
			UNLOCK(bAgentsLock);
			return FALSE;
		}
	}

	// Se si tratta dello stesso agente, aggiorniamo lo stato
	/*for(iter = agents->begin(); iter != agents->end(); iter++){
		if((*iter)->uAgentId == pAgent->uAgentId){
			(*iter)->uAgentStatus = pAgent->uAgentStatus;

			if((*iter)->pParams != pAgent->pParams){
				if((*iter)->pParams)
					delete[] (*iter)->pParams;

				(*iter)->pParams = pAgent->pParams;
				(*iter)->uParamLength = pAgent->uParamLength;
			}

			UNLOCK(bAgentsLock);
			return TRUE;
		}
	}*/

	newAgent = new(std::nothrow) AgentStruct;

	if(newAgent == NULL){
		UNLOCK(bAgentsLock);
		return FALSE;
	}

	memset(newAgent, 0x00, sizeof(AgentStruct));
	newAgent->uAgentId = pAgent->uAgentId;
	newAgent->uAgentStatus = pAgent->uAgentStatus;
	newAgent->uParamLength = pAgent->uParamLength;
	newAgent->pParams = pAgent->pParams;

	agents->push_back(newAgent);

	UNLOCK(bAgentsLock);
	return TRUE;
}

BOOL Status::AddEvent(EventStruct* pEvent) {
	EventStruct *newEvent = NULL;
	list<EventStruct *>::iterator iter;

	WAIT_AND_SIGNAL(bEventsLock);
	if(events == NULL){
		events = new(std::nothrow) list<EventStruct *>;

		if(events == NULL){
			UNLOCK(bEventsLock);
			return FALSE;
		}
	}

	/*
	// Se si tratta dello stesso evento, aggiorniamo lo stato
	for(iter = events->begin(); iter != events->end(); iter++){
		if((*iter)->uEventType == pEvent->uEventType){
			(*iter)->uActionId = pEvent->uActionId;

			if((*iter)->pParams != pEvent->pParams){
				if((*iter)->pParams)
					delete[] (*iter)->pParams;

				(*iter)->pParams = pEvent->pParams;
				(*iter)->uParamLength = pEvent->uParamLength;
			}

			UNLOCK(bActionsLock);
			return TRUE;
		}
	}*/

	newEvent = new(std::nothrow) EventStruct;

	if(newEvent == NULL){
		UNLOCK(bEventsLock);
		return FALSE;
	}

	memset(newEvent, 0x00, sizeof(EventStruct));
	newEvent->uEventType = pEvent->uEventType;
	newEvent->uActionId = pEvent->uActionId;
	newEvent->pParams = pEvent->pParams;
	newEvent->uParamLength = pEvent->uParamLength;

	events->push_back(newEvent);

	UNLOCK(bEventsLock);
	return TRUE;
}

BOOL Status::AddAction(MacroActionStruct* pAction) {
	MacroActionStruct *newAction = NULL;
	pActionStruct pList = NULL;
	list<MacroActionStruct *>::iterator iter;
	UINT i;

	WAIT_AND_SIGNAL(bActionsLock);
	if(actions == NULL){
		actions = new(std::nothrow) list<MacroActionStruct *>;

		if(actions == NULL){
			UNLOCK(bActionsLock);
			return FALSE;
		}
	}

	newAction = new(std::nothrow) MacroActionStruct;

	if(newAction == NULL){
		UNLOCK(bActionsLock);
		return FALSE;
	}

	memset(newAction, 0x00, sizeof(MacroActionStruct));
	newAction->uSubActions = pAction->uSubActions;
	newAction->pActions = new(std::nothrow) ActionStruct[pAction->uSubActions];

	if(newAction->pActions == NULL){
		delete newAction;
		UNLOCK(bActionsLock);
		return FALSE;
	}

	pActionStruct ptr = pAction->pActions;
	for(i = 0; i < newAction->uSubActions; i++, ptr++){
		newAction->pActions[i].uActionType = ptr->uActionType;
		newAction->pActions[i].uParamLength = ptr->uParamLength;
		newAction->pActions[i].pParams = ptr->pParams;
	}

	actions->push_back(newAction);

	UNLOCK(bActionsLock);
	return TRUE;
}

BOOL Status::AddConf(ConfStruct* pConf) {
	WAIT_AND_SIGNAL(bConfigLock);

	pConfStruct newConf = NULL;

	if(config == NULL){
		config = new(std::nothrow) list<ConfStruct *>;

		if(config == NULL){
			UNLOCK(bConfigLock);
			return FALSE;
		}
	}

	newConf = new(std::nothrow) ConfStruct;

	if(newConf == NULL){
		UNLOCK(bConfigLock);
		return FALSE;
	}

	newConf->uConfId = pConf->uConfId;
	newConf->uParamLength = pConf->uParamLength;
	newConf->pParams = pConf->pParams;

	config->push_back(newConf);
	UNLOCK(bConfigLock);
	return TRUE;
}

BOOL Status::SetDeviceStruct(DeviceStruct *dev) {
	if(device == NULL){
		device = new(std::nothrow) DeviceStruct;

		if(device == NULL)
			return FALSE;
	}

	memcpy(device, dev, sizeof(DeviceStruct));

	return TRUE;
}

// Questa funzione deve essere bloccante e va chiamata dopo aver invocato
// StopAgents() e StopEvents() da dentro Task.
BOOL Status::Clear() {
	list<AgentStruct *>::iterator iterAgent;
	list<EventStruct *>::iterator iterEvent;
	list<MacroActionStruct *>::iterator iterAction;
	list<ConfStruct *>::iterator iterConf;
	UINT i;

	if(agents == NULL && events == NULL && actions == NULL){
		return TRUE;
	}

	WAIT_AND_SIGNAL(bAgentsLock);
	if(agents){
		// Loopa qui dentro finche' tutti gli agenti non sono fermi
		for(iterAgent = agents->begin(); iterAgent != agents->end(); iterAgent++){
			if((*iterAgent)->uAgentStatus == AGENT_RUNNING){
				UNLOCK(bAgentsLock);
				Sleep(MAX_WAIT_TIME);
				iterAgent--;
				WAIT_AND_SIGNAL(bAgentsLock);
			}
		}

		agents->clear();
		delete agents;
		agents = NULL;
	}

	WAIT_AND_SIGNAL(bEventsLock);
	BOOL bAllStopped = TRUE;
	if(events){
		LOOP{
			// Loopa qui dentro finche' tutti i monitor non sono fermi
			for(iterEvent = events->begin(); iterEvent != events->end(); iterEvent++){
				if((*iterEvent)->uEventStatus == EVENT_RUNNING){
					UNLOCK(bEventsLock);
					Sleep(MAX_WAIT_TIME);
					bAllStopped = FALSE;
					WAIT_AND_SIGNAL(bEventsLock);
				}
			}

			if(bAllStopped)
				break;
			else
				bAllStopped = TRUE;
		}

		events->clear();
		delete events;
		events = NULL;
	}


	WAIT_AND_SIGNAL(bActionsLock);
	if(actions){
		// Loopa qui dentro finche' tutti i monitor non sono fermi
		for(iterAction = actions->begin(); iterAction != actions->end(); iterAction++){
			pActionStruct ptr = (*iterAction)->pActions;
			for(i = 0; i < (*iterAction)->uSubActions; i++, ptr++){
				if(ptr->pParams){
					delete[] ptr->pParams;
					ptr->pParams = NULL;
				}
			}

			delete[] (*iterAction)->pActions;

			delete *iterAction;
		}

		actions->clear();
		delete actions;
		actions = NULL;
	}

	WAIT_AND_SIGNAL(bConfigLock);
	if(config){
		// Rimuoviamo le opzioni di configurazione
		for(iterConf = config->begin(); iterConf != config->end(); iterConf++){
			if((*iterConf)->pParams)
				delete[] (*iterConf)->pParams;

			delete *iterConf;
		}

		config->clear();
		delete config;
		config = NULL;
	}

	if(device){
		delete device;
		device = NULL;
	}

	UNLOCK(bAgentsLock);
	UNLOCK(bEventsLock);
	UNLOCK(bActionsLock);
	UNLOCK(bConfigLock);
	return TRUE;
}

// CRITICAL - Questa funzione va chiamata SOLO all'interno di una sezione critica
UINT Status::QueryAgentStatus(UINT Type) {
	list<AgentStruct *>::iterator iter;

	if(agents == NULL){
		return AGENT_STOPPED;
	}

	for(iter = agents->begin(); iter != agents->end(); iter++){
		if((*iter)->uAgentId == Type){
			return (*iter)->uAgentStatus;
		}
	}

	return AGENT_STOPPED;
}

// CRITICAL - Questa funzione va chiamata SOLO all'interno di una sezione critica
UINT Status::QueryEventStatus(UINT Type, UINT uActionId) {
	list<EventStruct *>::iterator iter;

	if(events == NULL){
		return EVENT_STOPPED;
	}

	for(iter = events->begin(); iter != events->end(); iter++){
		if((*iter)->uEventType == Type && (*iter)->uActionId == uActionId){
			return (*iter)->uEventStatus;
		}
	}

	return EVENT_STOPPED;
}